#include "dictformatter.h"
#include "dict.h"

#include <QRegExp>

void tags(QString &text)
{
	text.replace('<', "&lt;");
	text.replace('>', "&gt;");
}

QString links(const QString &text)
{
	QString html = text;
	// replace html tags
	tags(html);

	int start = 0;
	while ((start = html.indexOf('{', start + 1)) >= 0) {
		int end = html.indexOf('}', start);
		// missing matching brace
		if (end < 0) break;

		int len = end - start;
		// too long for a link
		if (len > 240) continue;

		QString link = html.mid(start + 1, len - 1);
		// multiple lines inside braces, could be source code
		if (link.count('\n') > 1) continue;

		int pos;
		// skip external links
		if ((pos = link.indexOf('(')) >= 0 && link.indexOf(')') > pos
			&& len - pos > 5) {
				html.replace(start, len + 1, link);
				continue;
		}
		// format link
		html.replace(start, len + 1, QString("<a href=\"%1\">%2</a>")
			.arg(link.left(pos).simplified(), link));
	}
	return html;
}

QString freedict(const QString &text)
{
	QString html = text;

	QRegExp defn("^([^\n\\[]+)");
	html.replace(defn, "<dt><b>\\1</b>");

	QRegExp desc("\n\\s+(\\S+)");
	html.replace(desc, "<dd>\\1");

	return html;
}

QString beslov(const QString &text)
{
	QString html = text;

	QRegExp defn("^\\s+");
	html.replace(defn, "<dt class=\"defn\">");

	return html;
}

QString ozhegov(const QString &text)
{
	QString html = text;
	tags(html);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"defn\"><b>\\1</b><dd>");

	QRegExp exmpl("\n\\s+(E.g.)");
	html.replace(exmpl, "<dd>\\1");

	return html;
}

QString larousse(const QString &text)
{
	QString html = text;
	tags(html);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"defn\"><b>\\1</b><dt class=\"chap\">");

	QRegExp chap("\n\\s+([IVX]+\\.)");
	html.replace(chap, "<dt class=\"chap\">\\1");

	QRegExp sect("\n\\s+(\\d+\\.)");
	html.replace(sect, "<dt class=\"sect\">\\1");

	return html;
}

QString mueller(const QString &text)
{
	QString html = text;

	QRegExp abrev("_(\\S+\\.)");
	html.replace(abrev, "<i>\\1</i>");

	QRegExp chap("_(\\S+\\s)");
	html.replace(chap, "<dt class=\"chap\">\\1");

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt><b>\\1</b><dd class=\"abrv\">");

	QRegExp sect("\n\\s+(\\d+\\.)");
	html.replace(sect, "<dt class=\"sect\">\\1");

	QRegExp desc("\n\\s+(\\d+:\\s+)");
	html.replace(desc, "<dd>\\1");

	return html;
}

QString foldoc(const QString &text)
{
	QString html = links(text);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"defn\"><b>\\1</b>");

	QRegExp desc("\n\\s*\n");
	html.replace(desc, "<dd>\n");

	return html;
}

QString wordnet(const QString &text)
{
	QString html = links(text);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"defn\"><b>\\1</b><dd>");

	QRegExp desc("\n\\s+(\\d+:\\s+)");
	html.replace(desc, "<dd class=\"desc\">\\1");

	QRegExp desc2("\n\\s+(\\w+\\s\\d*:\\s+)");
	html.replace(desc2, "<dd class=\"desc\">\\1");

	QRegExp also("\n\\s+(\\[also:)");
	html.replace(also, "<dd>\\1");

	return html;
}

typedef QString (*formatfunc)(const QString &);
typedef QHash<QString, formatfunc> FormatMap;

class FormatTable
{
public:
	FormatTable();
	const FormatMap &map() const { return table; };

private:
	FormatMap table;
};

FormatTable::FormatTable()
{
	table["mueller"] = mueller;
	table["mueller24"] = mueller;
	table["beslov"] = beslov;
	table["ozhegov"] = ozhegov;
	table["larousse"] = larousse;
	table["foldoc"] = foldoc;
	table["wordnet"] = wordnet;
	table["wn"] = wordnet;
}

const FormatTable ftable;
const FormatMap &fmap = ftable.map();

QString DictFormatter::format(const DictResponse &response)
{
	QString html;
	formatfunc func;

	if (fmap.contains(response.database))
		func = fmap.value(response.database);
	else if (response.description.contains("Freedict"))
		func = freedict;
	else
		return words(response.defines);

	html = (QString("<div class=\"%1\">").arg(response.database));
	foreach (QString text, response.defines) {
// qDebug() << text;
		html += "<dl>";
		html += func(text);
		html += "</dl>";
	}
	html += "</div>";
	return html;
}

QString DictFormatter::words(const QStringList &list)
{
	QString html;
	foreach (QString text, list)
		html += words(text);
	return html;
}

QString DictFormatter::words(const QString &text)
{
	QString html;
	html += "<div class=\"define\">";
	html += links(text);
	html += "</div>";
	return html;
}
