#include "dictselection.h"
#include "dictsettings.h"

#include <QApplication>
#include <QClipboard>


DictSelection::DictSelection()
{
	timer = modifier = 0;
	local = remote = false;
	minLen = maxLen = 0;
	clipboard = QApplication::clipboard();
}

void DictSelection::setup(const DictConf &conf)
{
	local = conf.localYank;
	remote = conf.remoteYank;

	minLen = conf.yankMin;
	maxLen = conf.yankMax;
	modifier = (conf.modifierKey) ? Qt::ControlModifier : Qt::NoModifier;

	set(false);
	set(conf.yank);
}

void DictSelection::set(bool on)
{
	if (state == on)
		return;
	else
		state = on;

	if (on)
		connect(clipboard, SIGNAL(selectionChanged()), SLOT(selection()));
	else
		clipboard->disconnect(this);

#ifndef Q_WS_X11
	if (on)
		setTimeout(1000);
	else
		setTimeout(0);
#endif
}

static QString last;

QString DictSelection::current()
{
	return clipboard->text(QClipboard::Selection).left(maxLen);
}

Qt::KeyboardModifiers keyboardModifiers();

void DictSelection::selection()
{
	last = current();

	if (modifier && !(modifier & keyboardModifiers()))
		return;

	bool owner = clipboard->ownsSelection();
	if (!local && owner)
		return;
	if (!remote && !owner)
		return;

	if (last.length() < minLen || last.length() >= maxLen)
		return;

	emit selectionChanged(last);
}

#ifndef Q_WS_X11

void DictSelection::setTimeout(int timeout)
{
	if (timer)
		killTimer(timer);
	if (timeout) {
		last = current();
		timer = startTimer(timeout);
	}
}

void DictSelection::timerEvent(QTimerEvent *event)
{
	if (last != current()) {
		last = current();
		if (last.length() >= minLen && last.length() < maxLen)
			emit selectionChanged(last);
	}
}

#endif

#ifdef Q_WS_X11

#include <QX11Info>
#include <X11/XKBlib.h>

Qt::KeyboardModifiers keyboardModifiers()
{
	Qt::KeyboardModifiers keys = Qt::NoModifier;
	XkbStateRec state;

	XkbGetState(QX11Info::display(), XkbUseCoreKbd, &state);
// from qapplication x11
	if (state.base_mods & ShiftMask) keys |= Qt::ShiftModifier;
	if (state.base_mods & ControlMask) keys |= Qt::ControlModifier;
// from qkeymapper x11
	if (state.base_mods & Mod1Mask) keys |= Qt::AltModifier;
	if (state.base_mods & Mod4Mask) keys |= Qt::MetaModifier;

	return keys;
}

#else

Qt::KeyboardModifiers keyboardModifiers()
{
	return Qt::NoModifier;
}

#endif
